import stanza
from stanza.server import CoreNLPClient
from textalytics_core.entity_recognition import EntityRecognizer
from textalytics_core.resources import TextInput, EntityRecognizerOutput, Entity
import time

stanza.install_corenlp()

class StanzaEntityRecognizer(EntityRecognizer):
    def __init__(self):
        self.client = CoreNLPClient(
            annotators=['ner'],
            memory='4G',
            endpoint='http://localhost:9001',
            be_quiet=True)
        self.client.start()
        time.sleep(10)

    def recognize_entities(self, text_input: TextInput):
        entities = self.invoke_stanza_detect_entities(text_input.source_text)
        return EntityRecognizerOutput(entities=entities)

    def invoke_stanza_detect_entities(self, text_content):
        language = "en"
        stanza_response = self.client.annotate(text_content)
        return self.extract_entities(stanza_response)

    def extract_entities(self, stanza_response):
        entity_data = {}
        for sentence in stanza_response.sentence:
            for token in sentence.token:
                label = token.ner
                if label and label != "O":
                    ent_text = token.word
                    entity_data.setdefault(ent_text, {})
                    entity_data[ent_text]["entity"] = ent_text

                    entity_data[ent_text]["label"] = label

                    entity_data[ent_text]["pos_tag"] = token.pos
                    entity_data[ent_text]["start_offset"] = token.beginChar
                    entity_data[ent_text]["end_offset"] = token.endChar

        entities = []
        for key, values in entity_data.items():
            print(values)
            entities.append(Entity.parse_obj(values))
        return entities



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': '.'}

packages = \
['textalytics_spacy', 'textalytics_stanza']

package_data = \
{'': ['*']}

install_requires = \
['spacy>=3.1.2,<4.0.0',
 'stanza>=1.2.3,<2.0.0',
 'textalytics-core>=0.1.1,<0.2.0']

entry_points = \
{'console_scripts': ['test = scripts:test']}

setup_kwargs = {
    'name': 'textalytics-oss-adapters',
    'version': '0.1.0',
    'description': 'textalytics interface implementation for popular NLP open source libraries',
    'long_description': None,
    'author': 'Manoj Bharadwaj',
    'author_email': 'manoj@cloudcosmos.tech',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import spacy
from textalytics_core.entity_recognition import EntityRecognizer
from textalytics_core.resources import TextInput, EntityRecognizerOutput, Entity

nlp = spacy.load("en_core_web_sm")

class SpacyEntityRecognizer(EntityRecognizer):
    def recognize_entities(self, text_input: TextInput):
        entities = self.invoke_spacy_detect_entities(text_input.source_text)
        return EntityRecognizerOutput(entities=entities)

    def invoke_spacy_detect_entities(self, text_content):
        spacy_response = nlp(text_content)
        return self.extract_entities(spacy_response)

    def extract_entities(self, spacy_response):
        pos_tags = {}
        for token in spacy_response:
            pos_tags[token.text] = token.tag_

        entity_data = {}
        for spacy_entity in spacy_response.ents:
            ent_text = spacy_entity.text
            entity_data.setdefault(ent_text, {})
            entity_data[ent_text]["entity"] = ent_text

            entity_data[ent_text]["label"] = spacy_entity.label_

            entity_data[ent_text]["start_offset"] = spacy_entity.start_char
            entity_data[ent_text]["end_offset"] = spacy_entity.end_char

            if ent_text in pos_tags.keys():
                entity_data[ent_text]["pos_tag"] = pos_tags[ent_text]

        entities = []
        for key, values in entity_data.items():
            print(values)
            entities.append(Entity.parse_obj(values))
        return entities
